unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, AVRootIntf,
  StdCtrls;

type
  TForm1 = class(TForm, IApplication)
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private-Deklarationen }
    FLoader: IAVRootloader;
    FMode: TMode;
    FCompile: Boolean;
  protected
    { implements IApplication Interface}
    procedure ProcessMessages; stdcall;
    procedure Changed; stdcall;
    procedure Output(const Msg: WideString; Code: Integer); stdcall;

    function GetFLASHFileName: WideString; stdcall;
    function GetEEPROMFileName: WideString; stdcall;
    function GetACYFileName: WideString; stdcall;
    function GetPassword: WideString; stdcall;
    function GetBootSign: WideString; stdcall;
    function GetTimeouts: TTimeouts; stdcall;

    function OpenCommunication: ICOM; stdcall;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
procedure TForm1.ProcessMessages;
begin
  Application.ProcessMessages;
end;

procedure TForm1.Changed;
const
  sYesNo: array[Boolean] of String = ('No', 'Yes');
  sWire: array[Boolean] of String = ('2-Wire', '1-Wire');
begin
  if FMode <> FLoader.Mode then
  begin
    if not (FMode in [moConnected, moWorking]) and (FLoader.Mode in [moConnected, moWorking]) then
      with FLoader.Device do
      begin
        Memo1.Clear;
        Memo1.Lines.Add(Format('Connection                     : %6s', [sWire[FLoader.COM.EchoMode]]));
        Memo1.Lines.Add(Format('Device name                    : %s', [Name]));
        Memo1.Lines.Add(Format('Device signature               : %0.6x', [Signature]));
        Memo1.Lines.Add(Format('SRAM size                      : %6d Byte', [RamSize]));
        Memo1.Lines.Add(Format('EEPROM size                    : %6d Byte', [EepromSize]));
        Memo1.Lines.Add(Format('FLASH size                     : %6d Byte', [FlashSize]));
        Memo1.Lines.Add(Format('FLASH size for application     : %6d Byte', [AppFlashSize]));
        Memo1.Lines.Add(Format('FLASH pagesize                 : %6d Byte', [PageSize]));
        Memo1.Lines.Add(Format('Bootloader size                : %6d Byte', [FlashSize - AppFlashSize]));
        Memo1.Lines.Add(Format('Buffersize for data            : %6d Byte', [RamSize - PageSize + 24]));
        Memo1.Lines.Add(Format('SRAM start address             : %6d', [RamStartAddress]));
        Memo1.Lines.Add(Format('Bootloader version             : %6d', [Version]));
        Memo1.Lines.Add(Format('Use bootsection                : %6s', [sYesNo[UseBootSection]]));
        Memo1.Lines.Add(Format('Cryptography supported         : %6s', [sYesNo[Support and sfCrypt <> 0]]));
        if Support and sfCrypt <> 0 then
        begin
          Memo1.Lines.Add(Format('FLASH data must be encrypted   : %6s', [sYesNo[Support and sfCryptFlash <> 0]]));
          Memo1.Lines.Add(Format('EEPROM data must be encrypted  : %6s', [sYesNo[Support and sfCryptEeprom <> 0]]));
        end;
        if Info <> '' then
          Memo1.Lines.Add(Format('Info                           : %s', [Info]));
      end;

    FMode := FLoader.Mode;

    Button1.Enabled := FMode in [moDisconnected, moConnected];
    Button2.Enabled := FMode in [moDisconnected, moConnected];
    Button3.Enabled := FMode in [moDisconnected, moConnected];
    Button4.Enabled := FMode <> moWorking;
    case FMode of
      moDisconnected: Button4.Caption := 'Connect';
      moConnected: Button4.Caption := 'Disconnect';
      moConnecting: Button4.Caption := 'Abort connecting...';
      moWorking: Button4.Caption := 'Working..';
    end;
  end;
end;

procedure TForm1.Output(const Msg: WideString; Code: Integer);
begin
  Memo1.Lines.Add(Msg);
end;

function TForm1.GetFLASHFileName: WideString;
// wenn wir eine HEX/EPP Datei fr den Endkunden verschlsseln gebe den Pfad zur HEX datei zurck
// ansonsten wollen wir den AVR mit einer verschlsselten ACY Datei programmieren
begin
  if FCompile then Result := ExtractFilePath(ParamStr(0)) + 'test.hex'
    else Result := GetACYFilename;
end;

function TForm1.GetEEPROMFileName: WideString;
begin
  Result := '';//ExtractFilePath(ParamStr(0)) + 'test.epp';
end;

function TForm1.GetACYFileName: WideString;
begin
  Result := ExtractFilePath(ParamStr(0)) + 'test.acy';
end;

function TForm1.GetPassword: WideString;
// Zur Vereinfachung mal das ntige Passwort als Defaul Wert.
// Das mu natrlich bei der fertigen Version auf '' gesetzt werden !!
begin
  Result := InputBox('Password required', 'Password', '1470A1B30573D7DCE92F506A64CD6BBA');
end;

function TForm1.GetBootSign: WideString;
begin
  Result := 'BOOT';
end;

function TForm1.GetTimeouts: TTimeouts;
begin
  Result.Base := 50;
  Result.Erase := 20;
  Result.Flash := 25;
  Result.Eeprom := 20;
  Result.Buffer := 1;
  Result.RTSPulse := 0;
  Result.RTSInterval := 0;
end;

function TForm1.OpenCommunication: ICOM;
// hier knnte ein eigenes ICOM Interface erzeugt werden mit dem
// der Bootlaoder dann arbeitet. Dieses eigene Interface kann dann auf beliebige
// Hardware zugreifen, zb. USB Gerte etc.ppp.
begin
  Result := AVRootIntf.OpenCOM('\\.\COM2', Self);
// setze Baudrate
  Result.SetParams(115200);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FCompile := True;
  try
    FLoader.DoCompile(True, True);
  finally
    FCompile := False;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  FLoader.DoProgram(True, True);
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  FLoader.DoEraseFlash;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  case FLoader.Mode of
    moDisconnected: FLoader.DoConnect;
    moConnected: FLoader.DoDisconnect;
  else
    FLoader.DoAbort;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FLoader := AVRootIntf.OpenAVRootloader(Self);
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := FLoader.Mode = moDisconnected;
  if not CanClose then
    FLoader.DoDisconnect;
end;

end.
